package cn.iocoder.yudao.module.wealth.controller.admin.receiptItem;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.module.wealth.convert.receiptItem.ReceiptItemConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptItem.ReceiptItemDO;
import cn.iocoder.yudao.module.wealth.service.receiptItem.ReceiptItemService;
import cn.iocoder.yudao.module.wealth.vo.receipt.FinanceReceiptItemCancelVo;
import cn.iocoder.yudao.module.wealth.vo.receipt.FinanceReceiptItemVo;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.ReceiptItemBackVO;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.ReceiptItemCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.ReceiptItemUpdateReqVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 收款明细")
@RequestMapping("/ecw/receipt-item")
public class ReceiptItemController {

    @Resource
    private ReceiptItemService receiptItemService;

    @PostMapping("/create")
    @ApiOperation("创建收款明细")
    //@PreAuthorize("@ss.hasPermission('ecw:receipt-item:create')")
    public CommonResult<Long> createReceiptItem(@Valid @RequestBody ReceiptItemCreateReqVO createReqVO) {
        return success(receiptItemService.createReceiptItem(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新收款明细")
    //@PreAuthorize("@ss.hasPermission('ecw:receipt-item:update')")
    public CommonResult<Boolean> updateReceiptItem(@Valid @RequestBody ReceiptItemUpdateReqVO updateReqVO) {
        receiptItemService.updateReceiptItem(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除收款明细")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:receipt-item:delete')")
    public CommonResult<Boolean> deleteReceiptItem(@RequestParam("id") Long id) {
        receiptItemService.deleteReceiptItem(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得收款明细详情")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:receipt-item:query')")
    public CommonResult<ReceiptItemBackVO> getReceiptItem(@RequestParam("id") Long id) {
        ReceiptItemDO receiptItem = receiptItemService.getReceiptItem(id);
        return success(ReceiptItemConvert.INSTANCE.convert(receiptItem));
    }

    @GetMapping("/getReceivableItem")
    @ApiOperation("根据收款单ID获得收款明细列表")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<ReceiptItemBackVO>> getUpdateReceivableItemDetail(@RequestParam("id") Long id) {
        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper = new LambdaQueryWrapper();
        lambdaQueryWrapper.eq(ReceiptItemDO::getReceiptId, id);
        List<ReceiptItemDO> list = receiptItemService.list(lambdaQueryWrapper);
        return success(ReceiptItemConvert.INSTANCE.convertList(list));
    }

    @PostMapping("/financeReceiptItemWriteOff")
    @ApiOperation("收款明细核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> createFinanceReceiptItemWriteOff(@RequestBody FinanceReceiptItemVo financeReceiptVo) {
        receiptItemService.createFinanceReceiptItemWriteOff(financeReceiptVo);
        return success(true);
    }

    @PostMapping("/cancelFinanceReceiptItemWriteOff")
    @ApiOperation("取消收款明细核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinanceReceiptItemWriteOff(@RequestBody FinanceReceiptItemCancelVo financeReceiptVo) {
        receiptItemService.cancelFinanceReceiptItemWriteOff(financeReceiptVo);
        return success(true);
    }

    @PostMapping("/financeReceiptItemWriteOffNo")
    @ApiOperation("收款明细反核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> createFinanceReceiptItemWriteOffNo(@RequestBody FinanceReceiptItemVo financeReceiptVo) {
        receiptItemService.createFinanceReceiptItemWriteOffNo(financeReceiptVo);
        return success(true);
    }

    @PostMapping("/cancelFinanceReceiptItemWriteOffNo")
    @ApiOperation("取消收款明细反核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinanceReceiptItemWriteOffNo(@RequestBody FinanceReceiptItemCancelVo financeReceiptVo) {
        receiptItemService.cancelFinanceReceiptItemWriteOffNo(financeReceiptVo);
        return success(true);
    }

}
